<template>
	<div
		v-if="translationsRetrieved"
		:class="[{ 'mx-0 mb-1 col row p-2': isWidget }, styleClass]"
		class="mr-1"
	>
		<div class="col-12 px-0" v-if="title || intro">
			<h2 v-if="title">{{ title }}</h2>
			<p v-if="intro" v-html="intro"></p>
		</div>

		<div
			class="col-12 col-md p-0 validate-container"
			:class="[status($v.newsletterEmail), { 'mr-1': isWidget }]"
		>
			<floating-label
				:class="{ 'mb-1': !isWidget }"
				:config="{ label: $t('emailAddress') }"
				:model="$v.newsletterEmail.$model"
			>
				<input
					v-model="$v.newsletterEmail.$model"
					:class="status($v.newsletterEmail)"
					name="newsletter-firstname"
					type="email"
				/>
			</floating-label>
		</div>
		<template v-if="!isWidget">
			<transition name="slideFade">
				<div
					v-if="successMsg && hasEnrichment == 1"
					v-cloak
					class="col alert alert-success"
				>
					<span class="success-msg">{{ successMsg }}</span>
				</div>
			</transition>
			<transition name="slideFade">
				<div class="col alert alert-warning" v-if="errorMsg" v-cloak>
					<span class="error-msg">{{ errorMsg }}</span>
				</div>
			</transition>
		</template>
		<b-btn
			:class="{ 'col-auto px-3': isWidget }"
			class="btn col"
			@click="postEmail"
		>
			<svg v-if="updating" viewBox="0 0 56 56" width="100%" class="ok-loading">
				<g><circle cx="28" cy="28" :id="spinnerId" r="16" /></g>
			</svg>
			<template v-if="!updating">{{ $t("subscribe") }}</template>
			<template v-if="updating">{{ $t("subscribing") }}</template>
		</b-btn>

		<template v-if="isWidget">
			<transition name="slideFade">
				<div
					class="col-12 alert alert-success mt-1 mb-0"
					v-if="successMsg && hasEnrichment == 1"
					v-cloak
				>
					<span class="success-msg">{{ successMsg }}</span>
				</div>
			</transition>
			<transition name="slideFade">
				<div
					class="col-12 alert alert-warning mt-1 mb-0"
					v-if="errorMsg"
					v-cloak
				>
					<span class="error-msg">{{ errorMsg }}</span>
				</div>
			</transition>
		</template>

		<b-modal
			:id="modalRef"
			:ref="modalRef"
			:title="$t('enrichmentTitle')"
			:ok-title="okTitle"
			:cancel-title="$t('cancel')"
			:ok-only="enrichmentsSubmitted"
			class="newsletterEnrichmentModal"
			@ok="handleOk"
		>
			<img class="img-fluid" v-lazy="modalImg" :alt="$t('enrichmentTitle')" />

			<div class="p-1" v-if="!enrichmentsSubmitted">
				<div class="col alert alert-success" v-if="successMsg" v-cloak>
					<span class="success-msg">{{ successMsg }}</span>
				</div>
				<p>{{ $t("enrichmentIntro") }}</p>
				<form>
					<div
						class="col-12 mb-1 validate-container-radio"
						:class="status($v.enrichments.gender)"
					>
						<div class="row align-items-center mx-0">
							<small class="col-12 p-0 m-0 col-auto pr-1">{{
								$t("gender")
							}}</small>
							<b-form-radio-group v-model="$v.enrichments.gender.$model">
								<b-form-radio name="newsletter-gender" value="1">{{
									$t("male")
								}}</b-form-radio>
								<b-form-radio name="newsletter-gender" value="2">{{
									$t("female")
								}}</b-form-radio>
							</b-form-radio-group>
						</div>
					</div>
					<div
						class="col-12 validate-container mb-1"
						:class="status($v.enrichments.firstname)"
					>
						<floating-label
							:config="{ label: $t('firstname') }"
							:model="$v.enrichments.firstname.$model"
						>
							<input
								type="text"
								v-model="$v.enrichments.firstname.$model"
								name="newsletter-firstname"
								:class="status($v.enrichments.firstname)"
							/>
						</floating-label>
					</div>
					<div
						class="col-12 validate-container mb-1"
						:class="status($v.enrichments.lastname)"
					>
						<floating-label
							:config="{ label: $t('lastname') }"
							:model="$v.enrichments.lastname.$model"
						>
							<input
								type="text"
								v-model="$v.enrichments.lastname.$model"
								name="newsletter-lastname"
								:class="status($v.enrichments.lastname)"
							/>
						</floating-label>
					</div>
					<div
						class="col-12 mb-1 validate-container-date"
						:class="status($v.enrichments.dob)"
					>
						<div class="row align-items-center mx-0">
							<small class="col-12 p-0 m-0 col-auto pr-1">{{
								$t("dob")
							}}</small>
							<b-form-input
								class="col"
								type="date"
								v-model="$v.enrichments.dob.$model"
								:class="status($v.enrichments.dob)"
								name="newsletter-dob"
							></b-form-input>
						</div>
					</div>
					<div
						class="col-12 mb-1 validate-container"
						:class="status($v.enrichments.password)"
						v-if="showPassword"
					>
						<floating-label
							:config="{ label: $t('password') }"
							:model="$v.enrichments.password.$model"
						>
							<input
								type="password"
								v-model="$v.enrichments.password.$model"
								:class="status($v.enrichments.password)"
								name="newsletter-password"
							/>
						</floating-label>
					</div>
					<div class="row m-0 px-1" v-if="showPassword">
						<div class="col alert alert-info">
							<span class="info-msg">{{ $t("accountDetected") }}</span>
						</div>
					</div>
				</form>
			</div>
			<div v-else class="p-1">
				<h2>{{ $t("thankYou") }}</h2>
				<p>{{ $t("thankYouText") }}</p>
			</div>
		</b-modal>
	</div>
</template>

<script>
	module.exports = {
		data() {
			return {
				enrichments: {
					dob: "",
					firstname: "",
					gender: "",
					lastname: "",
					password: "",
				},
				enrichmentsSubmitted: false,
				errorMsg: false,
				hasEnrichment: false,
				modalImg: skinUrl + "images/letsgetpersonal.jpg",
				modalRef: "",
				newsletterEmail: "",
				showPassword: false,
				spinnerId: "",
				successMsg: false,
				translations: {},
				translationsRetrieved: false,
				updating: false,
			};
		},
		props: {
			intro: {
				type: String,
				default: "",
			},
			isWidget: {
				type: Boolean,
				default: false,
			},
			styleClass: {
				type: String,
				default: "",
			},
			title: {
				type: String,
				default: "",
			},
		},
		computed: {
			okTitle() {
				return !this.enrichmentsSubmitted ? this.$t("submit") : this.$t("ok");
			},
		},
		components: {
			FloatingLabel:
				"url:" +
				jsBaseUrl +
				"vue/ok_components/general/FloatingLabel.vue?v=" +
				smsGeneralJsVersion,
		},
		methods: {
			$t(val) {
				return this.translations[val];
			},
			handleOk(bvModalEvt) {
				bvModalEvt.preventDefault();

				if (!this.enrichmentsSubmitted) {
					this.$v.enrichments.$touch();

					if (!this.$v.enrichments.$invalid) {
						this.submitEnrichments();
					}
				} else {
					this.$refs[this.modalRef].hide();
				}
			},
			postEmail() {
				this.errorMsg = "";
				if (this.updating) return true;
				if (
					this.$v.newsletterEmail.$invalid ||
					this.newsletterEmail.trim() == ""
				) {
					this.errorMsg = this.translations.invalidEmailAddress;
					return;
				}
				this.updating = true;

				const url = `https://a.klaviyo.com/client/subscriptions/?company_id=Ut4aBc`;
				const body = JSON.stringify({
					data: {
						type: "subscription",
						attributes: {
							custom_source: "Website Footer or Widget",
							profile: {
								data: {
									type: "profile",
									attributes: { email: this.newsletterEmail.trim() },
								},
							},
						},
						relationships: { list: { data: { type: "list", id: "RNWSnj" } } },
					},
				});

				this.$http
					.post(url, body, {
						headers: {
							revision: moment().format("YYYY-MM-DD"),
							"content-type": "application/json",
						},
					})
					.then(response => {
						if (response.ok) {
							this.$refs[this.modalRef].show();
						}
					})
					.catch(err => {
						this.errorMsg = err;
						console.error(err);
					})
					.finally(() => (this.updating = false));
			},
			status(validation) {
				return {
					error: validation.$error,
					dirty: validation.$dirty,
				};
			},
			submitEnrichments() {
				this.errorMsg = "";
				if (this.updating) return true;
				if (
					this.$v.newsletterEmail.$invalid ||
					this.newsletterEmail.trim() == ""
				) {
					this.errorMsg = this.translations.invalidEmailAddress;
					return;
				}
				this.updating = true;

				const url = `https://a.klaviyo.com/client/profiles/?company_id=Ut4aBc`;
				const body = JSON.stringify({
					data: {
						type: "profile",
						attributes: {
							email: this.newsletterEmail.trim(),
							first_name: this.enrichments.firstname,
							last_name: this.enrichments.lastname,
							properties: {
								Birthday: this.enrichments.dob,
								Gender: this.enrichments.gender,
							},
						},
					},
				});

				this.$http
					.post(url, body, {
						headers: {
							revision: moment().format("YYYY-MM-DD"),
							"content-type": "application/json",
						},
					})
					.then(response => {
						if (response.ok) {
							this.enrichmentsSubmitted = true;
						}
					})
					.catch(err => {
						this.errorMsg = err;
						console.error(err);
					})
					.finally(() => (this.updating = false));
			},
			getContent() {
				this.$http.get(newsletterData.translationUrl, {}).then(response => {
					this.translations = JSON.parse(response.body);
					this.translationsRetrieved = true;
				});
			},
		},
		watch: {
			newsletterEmail() {
				this.errorMsg = "";
			},
		},
		validations: {
			newsletterEmail: {
				required,
				email,
			},
			enrichments: {
				dob: {
					isLogicalDate: dob => {
						const today = moment();
						const diff = today.diff(moment(dob), "years");
						return diff >= 0 && diff <= 150;
					},
				},
				gender: {},
				firstname: {},
				lastname: {},
				password: {
					required: requiredIf(() => {
						return this.showPassword;
					}),
				},
			},
		},
		mounted() {
			this.getContent();
			let date = new Date();
			let random = date.getMilliseconds() + Math.random();
			this.modalRef = "newsletterEnrichmentModal" + random;
			this.spinnerId = "spinner" + random;
		},
	};
</script>

<style scoped>
	.success-page-widget {
		padding: 0 !important;
		margin-bottom: 0 !important;
	}
	.content-widget {
		border: 1px solid #e6e6e6;
	}
</style>
